#include "Space.h"
#include "Site.h"
#include "MASS.h"
#include <stack>
using namespace MMASS;
using std::stack;

Space::~Space() { }

spSite Space::createSite(const string &name_) throw (logic_error, invalid_argument) {
	if (ready)
		throw logic_error("Space::createSite(const string &name_) -"
			" name_: "+name_+"\n"
			"This space (of the MASS \""+mass.lock()->getName()+"\")"
			" is alredy finished, can't create sites in a finished space.\n");

	if (this->hasSite(name_))
		throw invalid_argument("Space::createSite(const string &name_) -"
			" name_: "+name_+"\n"
			"A site with this name already exists in the space of the "
			"MASS \""+mass.lock()->getName()+"\".\n");

	spSite site = Site::createSite(name_);
	sites[name_] = site;
	site->setSpace(self);

	return site;
}

spSite Space::createSite(const string &name_, spSite_set &adjSites) throw (logic_error, invalid_argument) {
	if (ready)
		throw logic_error("Space::createSite(const string &name_, spSite_set &adjSites) -"
			" name_: "+name_+"\n"
			"This space (of the MASS \""+mass.lock()->getName()+"\")"
			" is alredy finished, can't create sites in a finished space.\n");

	spSite nullSite;
	spSite_set::iterator i = adjSites.find(nullSite);
	if (i != adjSites.end())
		throw invalid_argument("Space::createSite(const string &name_, spSite_set &adjSites) -"
			" name_: "+name_+"\n"
			"adjSites has a null site. A site can't be adjacent to a null site.\n");

	spSite_set::iterator end = adjSites.end();
	for (i = adjSites.begin(); i != end; ++i) {
		if ((*i)->getSpace() != self.lock())
			throw invalid_argument("Space::createSite(const string &name_, spSite_set &adjSites) -"
				" name_: "+name_+"\n"
				"The site \""+(*i)->getName()+"\" in adjSites is in space from MASS \""
				+(*i)->getSpace()->getMASS()->getName()+"\", this space is from MASS \""
				+mass.lock()->getName()+"\". A site can't be adjacent to a site in"
				" a different space.\n");
	}

	spSite site;
	try {
		site = this->createSite(name_);
	} catch (invalid_argument &e) {
		throw invalid_argument(string("Space::createSite(const string &name_, spSite_set &adjSites) -"
			" name_: "+name_+"\n"
			"<-- ")+e.what());
	}

	try {
		for (i = adjSites.begin(); i != end; ++i)
			site->addAdjacentSite((*i));
	} catch (Site::invalid_argument &e) {
		throw invalid_argument(string("Space::createSite(const string &name_, spSite_set &adjSites) -"
			" name_: "+name_+"\n"
			"<-- ")+e.what());
	}

	return site;
}

const string& Space::getName() const {
	return name;
}

const spSpace Space::getSelf() const {
	return self.lock();
}

const bool Space::hasMASS() const {
	return mass.lock();
}

const spMASS Space::getMASS() const {
	return mass.lock();
}

const bool Space::hasSite() const {
	return (!sites.empty());
}

const bool Space::hasSite(const string &name_) const {
	return (sites.count(name_) > 0);
}

const bool Space::hasSite(spSite site) const {
	return (this->hasSite(site->getName()));
}

const size_t Space::numOfSites() const {
	return sites.size();
}

spSite Space::getSite(const string &name_) throw (logic_error, invalid_argument){
	if (!this->hasSite())
		throw logic_error("Space::getSite(const string &name_) - "
			"name_: "+name_+"\n"
			"The space from MASS \""+mass.lock()->getName()+ "\""
			" doesn't have a site.\n");

	if (!this->hasSite(name_))
		throw invalid_argument("Space::getSite(const string &name_) - "
			"name_: "+name_+"\n"
			"The space from MASS \""+mass.lock()->getName()+ "\""
			" doesn't have a site with this name.\n");

	return sites[name_];
}

spSite_map::const_iterator Space::beginSites() const {
	return sites.begin();
}

spSite_map::const_iterator Space::endSites() const {
	return sites.end();
}

const bool Space::isReady() const {
	return ready;
}

void Space::finish() throw (logic_error) {
	try {
		this->checkGraph();
	} catch (logic_error &e) {
		throw logic_error(string("Space::finish() -\n"
			"<-- ")+e.what());
	}

	spSite_map::const_iterator site_i, s_end = this->sites.end();
	for (site_i = this->sites.begin(); site_i != s_end; ++site_i)
		(*site_i).second->calculateDistances();

	ready = true;
}

Space::Space() {
	ready = false;
}

spSpace Space::createSpace() {
	spSpace space(new Space);
	space->setSelfPointer(space);

	return space;
}

void Space::setSelfPointer(const wpSpace self_) {
	self = self_;
}

void Space::setMASS(wpMASS mass_) {
	mass = mass_;
}

void Space::checkGraph() throw (logic_error) {
	unsigned int numOfConnectedComponents = 0;

	if(sites.empty())
		throw logic_error(string("Space::checkGraph() - "
					"this space (from MASS \"")+mass.lock()->getName()+
					string("\") has a empty set of sites."));

	spSite_map::iterator site_i, end = sites.end();

	for (site_i = sites.begin(); site_i != end; ++site_i) {
		if (!(*site_i).second->wasVisited()) {
			if (numOfConnectedComponents >= 1)
				throw logic_error(string("Space::checkGraph() - "
					"this space (from MASS \"")+mass.lock()->getName()+
					string("\") has a set of sites that doesn't form "
					"a connected graph."));

			dfsRcheck((*site_i).second);
			numOfConnectedComponents++;
		}
	}
}

void Space::dfsRcheck(spSite site) {
	stack<wpSite> st;
	st.push(site);

	while (!st.empty()) {
		wpSite s = st.top();
		st.pop();

		s.lock()->setVisited(true);

		wpSite_map::const_iterator adjSite_i, end = s.lock()->endAdjacentSites();
		for (adjSite_i = s.lock()->beginAdjacentSites(); adjSite_i != end; ++adjSite_i) {
			if (!((*adjSite_i).second).lock()->wasVisited())
				st.push((*adjSite_i).second);
		}
	}

	/*
	fprintf(fp, "dfs check %s -> ", site->getName().c_str()); fflush(fp);

	site->setVisited(true);
	
	wpSite_map::const_iterator adjSite_i, end = site->endAdjacentSites();
	for (adjSite_i = site->beginAdjacentSites(); adjSite_i != end; ++adjSite_i) {
		fprintf(fp, "%s (%d)\n", (*adjSite_i).second.lock()->getName().c_str(), ((*adjSite_i).second).lock()->wasVisited()); fflush(fp);
		if (!((*adjSite_i).second).lock()->wasVisited())
			dfsRcheck(((*adjSite_i).second).lock());
	}
	*/
}
