#ifndef AGENT_H
#define AGENT_H

#ifdef MMASSDLL_EXPORTS
#define MMASSDLL_API __declspec(dllexport)
#else
#define MMASSDLL_API __declspec(dllimport)
#endif

#include "MMASSDefs.h"
#include <OgreNoMemoryMacros.h>
#include <string>
#include <stdexcept>
#include <map>

namespace MMASS {
	using std::string;
	using std::map;

	class MMASSDLL_API Agent {
	public:
		class logic_error : public std::logic_error {
		public:
			logic_error(const string& msg = "") : std::logic_error(msg) {}
		};

		class invalid_argument : public std::invalid_argument {
		public:
			invalid_argument(const string& msg = "") : std::invalid_argument(msg) {}
		};

	protected:
		wpSite  site;

	public:
		unsigned long action_counter;
		unsigned long action_denied_counter;
		unsigned long action_accepted_counter;

		class AgentFactory {
			virtual Agent* create(const string &name_) const = 0;
			friend Agent;
		};

		virtual ~Agent() = 0;
		
		const string& getName() const;

		const spAgent getSelf() const;

		const bool hasSite() const;
		spSite getSite() const;
		void setSite(spSite site_) throw (logic_error, invalid_argument);

		const spMASS getMASS() const;

		virtual const string getTypeName() = 0;

		unsigned long getCoefficient(const string &fieldIdentifier);
		const double getThreshold(const string &fieldIdentifier);

	protected:
		Agent(const string &name_);

		spAction getPossibleAction();
		void setPossibleAction(spAction action);

		spActionInfo_set::const_iterator beginDeniedActions();
		spActionInfo_set::const_iterator endDeniedActions();
		bool hasDeniedAction(const string &actions_identifier);

		void setCoefficient(const string &fieldIdentifies, unsigned long coefficient);
		void setThreshold(const string &fieldIdentifier, const double threshold);

		unsigned long getTurn();
		
	private:
		const string                     name;
		wpAgent                          self;
		wpMASS                           mass;
		spAction                         action;
		spAction                         possibleAction;
		spAction                         suspendedAction;
		spActionInfo_set                 deniedActions;
		map<const string, unsigned long> fieldsCoefficients;
		map<const string, double>        fieldsThresholds;
		bool                             action_accepted;
		bool                             action_performed;
		bool                             finish_thread;
		unsigned long                    turn;

		static spAgent createAgent(AgentFactory &factory, const string &name_);

		void setSelf(wpAgent self_);

		void setMASS(const wpMASS mass_);

		void setSiteToNull();

		spAction collectAction();
		void setAction(spAction action_);
		spAction getAction();
		void setActionToNull();

		void addDeniedAction(spActionInfo action);
		void clearDeniedActions();

		void setActionAccepted(bool b);
		bool actionAccepted();

		bool actionPerformed();
		void setActionPerformed(bool b);

		bool finishThread();
		void setFinishThread(bool b);

		void nextTurn();

		spAgentInfo createAgentInfo();

		virtual void actionSelect(spContext localContext) = 0;
		virtual const bool agreeReaction(spAgentInfo_set &involvedAgents) = 0;
		void performAction();
		void notifyFailure();

		// virtual void printState() = 0;

		/* Hiding methods. */
		Agent(const Agent&);
		Agent& operator=(const Agent&);

		friend class Context;
		friend class Site;
		friend class MASS;
		friend class CMMASS;
		//friend MMASSDLL_API void* _agentBehaviourThread(void *arg);
		friend MMASSDLL_API void* _environmentBehaviourThread(void *arg);
		friend MMASSDLL_API bool _manage(spAgent agent, spAction action);
		friend MMASSDLL_API bool _transportManagement(spAgent agent, spAction action);
		friend MMASSDLL_API bool _emitManagement(spAgent agent, spAction action);
		friend MMASSDLL_API bool _reactionManagement(spAgent agent, spAction action);
		friend MMASSDLL_API bool _triggerManagement(spAgent agent, spAction action);
		friend MMASSDLL_API void _performActions(spAgent_set &agents);
	};

	template<class StateClass>
	class CAgent : public Agent {
	public:
		boost::shared_ptr<StateClass> getState() { return state; }

		static boost::shared_ptr<StateClass> extractState(spAgent agent_) {
			Agent *aPtr = agent_.get();
			CAgent *caPtr = dynamic_cast<CAgent*>(aPtr);
			return caPtr->getState();
		}

		~CAgent<StateClass>(){}

	protected:
		boost::shared_ptr<StateClass> state;

		CAgent(const std::string &name_, boost::shared_ptr<StateClass> state_) : Agent(name_), state(state_) { }
	};

	class MMASSDLL_API AgentInfo {
	public:
		const string& getAgentName();
		const string& getTypeName();
		const string& getSiteName();

	protected:
		const string agentName;
		const string typeName;
		const string siteName;

	private:
		AgentInfo(const string &agentName_, const string &typeName_, const string &siteName_);
		static spAgentInfo createAgentInfo(const string &agentName_, const string &typeName_, const string &siteName_);

		friend class Agent;
	};
}

#endif