#include "TestSite.h"
#include "../MMASSdll/Space.h"
#include "../MMASSdll/MMASS.h"
using namespace MMASS;

void TestSite::run() {
	usingSiteTests();
}

void TestSite::usingSiteTests() {
	spMMASS mmass = CMMASS::getSingleton();
	spSpace space = mmass->getMASS("mass1")->getSpace();

	try { /* Making one site adjacent to another. */
		spSite site11 = space->getSite("site1.1");
		test_(!site11->hasAdjacentSites());
		test_(!site11->isAdjacentTo("site1.3"));
		size_t nAdjSites11 = site11->numOfAdjacentSites();

		spSite site13 = space->getSite("site1.3");
		test_(site13->hasAdjacentSites());
		test_(!site13->isAdjacentTo("site1.3"));
		size_t nAdjSites13 = site13->numOfAdjacentSites();

		site11->addAdjacentSite(site13);

		test_(site11->hasAdjacentSites());
		test_(site11->isAdjacentTo("site1.3"));
		test_(site11->isAdjacentTo(site13));
		test_(site11->numOfAdjacentSites() == ++nAdjSites11);
		test_(site13->isAdjacentTo("site1.1"));
		test_(site13->isAdjacentTo(site11));
		test_(site13->numOfAdjacentSites() == ++nAdjSites13);
	} catch (std::exception &e) {
		fail_(string("An unexpected exception have occured:\n")
			+ e.what());
	}

	try { /* Adding sites that are already adjacents
			doesn't have any effect. */
		spSite site11 = space->getSite("site1.1");
		size_t nAdjSites11 = site11->numOfAdjacentSites();

		spSite site13 = space->getSite("site1.3");
		size_t nAdjSites13 = site13->numOfAdjacentSites();

		site11->addAdjacentSite(site13);

		test_(site11->isAdjacentTo("site1.3"));
		test_(site11->numOfAdjacentSites() == nAdjSites11);
		test_(site13->isAdjacentTo("site1.1"));
		test_(site13->numOfAdjacentSites() == nAdjSites13);
	} catch (std::exception &e) {
		fail_(string("An unexpected exception have occured:\n")
			+ e.what());
	}

	try { /* Can't have a site adjacent to null. */
		spSite site11 = space->getSite("site1.1");
		spSite nullSite;

		site11->addAdjacentSite(nullSite);

		fail_("Trying to make a site adjacent to null. "
			"It should have thrown an exception.");
	} catch (Site::invalid_argument &) {
		succeed_();
	} catch (std::exception &e) {
		fail_(string("An unexpected exception have occured:\n")
			+ e.what());
	}

	try { /* Can't make one site adjacent to another
			site from a different space. */
		spSite site12 = space->getSite("site1.2");
		test_(!site12->hasAdjacentSites());
		test_(!site12->isAdjacentTo("site1.1"));
		test_(site12->numOfAdjacentSites() == 0);

		spSpace space2 = mmass->getMASS("mass2")->getSpace();
		spSite site11 = space2->getSite("site1.1");
		test_(!site11->hasAdjacentSites());
		test_(!site11->isAdjacentTo("site1.2"));
		test_(site11->numOfAdjacentSites() == 0);

		site12->addAdjacentSite(site11);

		fail_("Trying to make a site adjacent to another site"
			" from a different space. It should have thrown "
			"an exception.");
	} catch (Site::invalid_argument &) {
		succeed_();

		spSite site12 = space->getSite("site1.2");
		test_(!site12->hasAdjacentSites());
		test_(!site12->isAdjacentTo("site1.1"));
		test_(site12->numOfAdjacentSites() == 0);

		spSpace space2 = mmass->getMASS("mass2")->getSpace();
		spSite site11 = space2->getSite("site1.1");
		test_(!site11->hasAdjacentSites());
		test_(!site11->isAdjacentTo("site1.2"));
		test_(site11->numOfAdjacentSites() == 0);
	} catch (std::exception &e) {
		fail_(string("An unexpected exception have occured:\n")
			+ e.what());
	}

	try { /* Making one site adjacent to a set of sites. */
		spSite site11 = space->getSite("site1.1");
		test_(!site11->isAdjacentTo("site1.2"));

		spSite_set set;
		set.insert(space->getSite("site1.2"));
		set.insert(space->getSite("site1.7"));
		test_(!space->getSite("site1.2")->isAdjacentTo("site1.1"));
		test_(!space->getSite("site1.7")->isAdjacentTo("site1.1"));
		test_(!space->getSite("site1.7")->isAdjacentTo("site1.2"));
	
		site11->addAdjacentSites(set);
		test_(site11->isAdjacentTo("site1.2"));
		test_(space->getSite("site1.2")->isAdjacentTo("site1.1"));
		test_(space->getSite("site1.7")->isAdjacentTo("site1.1"));
		test_(!space->getSite("site1.7")->isAdjacentTo("site1.2"));
	} catch (std::exception &e) {
		fail_(string("An unexpected exception have occured:\n")
			+ e.what());
	}

	try { /* Making one site adjacent to a set of sites
			containing a NULL. */
		spSite nullSite;
		spSite site12 = space->getSite("site1.2");
		test_(!site12->isAdjacentTo("site1.7"));

		spSite_set set;
		set.insert(space->getSite("site1.7"));
		set.insert(nullSite);
		test_(!space->getSite("site1.7")->isAdjacentTo("site1.2"));
	
		site12->addAdjacentSites(set);
		
		fail_("The set has a null site. "
			"It should have thrown an exception.");
	} catch (Site::invalid_argument &) {
		succeed_();

		spSite site12 = space->getSite("site1.2");
		test_(!site12->isAdjacentTo("site1.7"));
		test_(!space->getSite("site1.7")->isAdjacentTo("site1.2"));
	} catch (std::exception &e) {
		fail_(string("An unexpected exception have occured:\n")
			+ e.what());
	}

	try { /* Making one site adjacent to a set of sites
			containing a site from another space. */
		spSpace space2 = mmass->getMASS("mass2")->getSpace();
		spSite site12 = space->getSite("site1.2");
		test_(!site12->isAdjacentTo("site1.7"));

		spSite_set set;
		set.insert(space->getSite("site1.7"));
		set.insert(space2->getSite("site1.1"));
		test_(!space2->getSite("site1.1")->isAdjacentTo(site12));
	
		site12->addAdjacentSites(set);
		
		fail_("The set has a null site. "
			"It should have thrown an exception.");
	} catch (Site::invalid_argument &) {
		succeed_();

		spSpace space2 = mmass->getMASS("mass2")->getSpace();
		spSite site12 = space->getSite("site1.2");

		test_(!site12->isAdjacentTo("site1.7"));
		test_(!space->getSite("site1.7")->isAdjacentTo("site1.2"));
		test_(!space2->getSite("site1.1")->isAdjacentTo(site12));
	} catch (std::exception &e) {
		fail_(string("An unexpected exception have occured:\n")
			+ e.what());
	}

	try { /* Making one site adjacent to a empty set of sites
		  doesn't have any effect. */
		spSite site11 = space->getSite("site1.1");
		test_(site11->isAdjacentTo("site1.2"));
		test_(site11->isAdjacentTo("site1.3"));
		test_(!site11->isAdjacentTo("site1.4"));
		test_(!site11->isAdjacentTo("site1.5"));
		test_(!site11->isAdjacentTo("site1.6"));
		test_(site11->isAdjacentTo("site1.7"));

		spSite_set set;
		site11->addAdjacentSites(set);

		test_(site11->isAdjacentTo("site1.2"));
		test_(site11->isAdjacentTo("site1.3"));
		test_(!site11->isAdjacentTo("site1.4"));
		test_(!site11->isAdjacentTo("site1.5"));
		test_(!site11->isAdjacentTo("site1.6"));
		test_(site11->isAdjacentTo("site1.7"));
	} catch (std::exception &e) {
		fail_(string("An unexpected exception have occured:\n")
			+ e.what());
	}

	/*
	Agent007::Agent007Factory factory;

	try {
		spSite site1 = Site::createSite("site1");

		test_(!site1->hasAgent());
		test_(site1->numOfAdjacentSites() == 0);
		test_(site1->beginAdjacentSites() == site1->endAdjacentSites());
		test_(site1->getName() == "site1");
		test_(site1->getSelf() == site1);
	} catch (Site::invalid_argument_site& e) {
		fail_(string("Shouldn't have thrown exception. ") + e.what());
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	try {
		spSite_set sc1;

		spSite adjSite2 = Site::createSite("adjSite2_1");
		sc1.insert(adjSite2);

		adjSite2 = Site::createSite("adjSite2_2");
		sc1.insert(adjSite2);

		spSite site2 = Site::createSite("site2", sc1);

		test_(!site2->hasAgent());
		test_(site2->numOfAdjacentSites() == 2);
		test_(site2->beginAdjacentSites() != site2->endAdjacentSites());
		test_(site2->getName() == "site2");
		test_(site2->getSelf() == site2);

		spSite_set::iterator i = sc1.begin(), end = sc1.end();
		while (i != end) {
			test_((*i)->isAdjacentTo(site2));
			test_(site2->isAdjacentTo((*i)));
			++i;
		}
	} catch (Site::invalid_argument_site& e) {
		fail_(string("Shouldn't have thrown exception. ") + e.what());
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	try {
		spState agentState3(new State);
		spAgent agent3 = Agent007::createAgent(factory, "agent3", agentState3);

		spSite site3 = Site::createSite("site3", agent3);

		test_(site3->hasAgent());
		test_(site3->getAgent() == agent3);
		test_(agent3->getSite() == site3);
		test_(site3->numOfAdjacentSites() == 0);
		test_(site3->beginAdjacentSites() == site3->endAdjacentSites());
		test_(site3->getName() == "site3");
		test_(site3->getSelf() == site3);
	} catch (Site::invalid_argument_site& e) {
		fail_(string("Shouldn't have thrown exception. ") + e.what());
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	try {
		spSite_set sc4;

		spSite adjSite4 = Site::createSite("adjSite4_1");
		sc4.insert(adjSite4);

		adjSite4 = Site::createSite("adjSite4_2");
		sc4.insert(adjSite4);

		spState agentState4(new State);
		spAgent agent4 = Agent007::createAgent(factory, "agent4", agentState4);

		spSite site4 = Site::createSite("site4", agent4, sc4);

		test_(site4->hasAgent());
		test_(site4->getAgent() == agent4);
		test_(agent4->getSite() == site4);
		test_(site4->numOfAdjacentSites() == 2);
		test_(site4->beginAdjacentSites() != site4->endAdjacentSites());
		test_(site4->getName() == "site4");
		test_(site4->getSelf() == site4);

		spSite_set::iterator i = sc4.begin(), end = sc4.end();
		while (i != end) {
			test_((*i)->isAdjacentTo(site4));
			test_(site4->isAdjacentTo((*i)));
			++i;
		}
	} catch (Site::invalid_argument_site& e) {
		fail_(string("Shouldn't have thrown exception. ") + e.what());
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	try {
		spSite site5 = Site::createSite("site5");
		spSite site5_1 = Site::createSite("site5");

		fail_("Should have thrown exception.");
	} catch (Site::invalid_argument_site&) {
		succeed_();
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	try {
		spState agentState6(new State);
		spAgent agent6 = Agent007::createAgent(factory, "agent6", agentState6);

		spSite site6 = Site::createSite("site6");
		spSite site6_1 = Site::createSite("site6", agent6);

		fail_("Should have thrown exception.");
	} catch (Site::invalid_argument_site&) {
		succeed_();
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}
	try {
		spSite_set sc7;

		spSite adjSite7 = Site::createSite("adjSite7");
		sc7.insert(adjSite7);

		spSite site7 = Site::createSite("site7");
		spSite site7_1 = Site::createSite("site7", sc7);

		fail_("Should have thrown exception.");
	} catch (Site::invalid_argument_site&) {
		succeed_();
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	try {
		spSite_set sc8;

		spSite adjSite8 = Site::createSite("adjSite8");
		sc8.insert(adjSite8);

		spState agentState8(new State);
		spAgent agent8 = Agent007::createAgent(factory, "agent8", agentState8);

		spSite site8 = Site::createSite("site8");
		spSite site8_1 = Site::createSite("site8", agent8, sc8);

		fail_("Should have thrown exception.");
	} catch (Site::invalid_argument_site&) {
		succeed_();
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	try {
		spSite agentSite9 = Site::createSite("agentSite9");
		spState agentState9(new State);
		spAgent agent9 = Agent007::createAgent(factory, "agent9", agentState9, agentSite9);

		spSite site9 = Site::createSite("site6", agent9);
		fail_("Should have thrown exception.");
	} catch (Site::invalid_argument_site&) {
		succeed_();
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}
	try {
		spSite_set sc10;

		spSite adjSite10 = Site::createSite("adjSite10");
		sc10.insert(adjSite10);

		spSite agentSite10 = Site::createSite("agentSite10");
		spState agentState10(new State);
		spAgent agent10 = Agent007::createAgent(factory, "agent10", agentState10, agentSite10);

		spSite site10 = Site::createSite("site10", agent10, sc10);

		fail_("Should have thrown exception.");
	} catch (Site::invalid_argument_site&) {
		succeed_();
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}
	*/
}
/*
void TestSite::addingObjectsTests() {
	spSite site = Site::createSite("Site");
	test_(site->getName() == "Site");

	addingAgentTest(site);
	addingSitesTests(site);
}

void TestSite::addingAgentTest(spSite site) {
	Agent007::Agent007Factory factory;

	try {
		spAgent agent1;
		site->setAgent(agent1);

		test_(!site->hasAgent());
		test_(site->numOfAdjacentSites() == nSites);
	} catch (Site::invalid_argument_site& e) {
		fail_(string("Shouldn't have thrown an exception. ") + e.what());
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	try {
		spState aState(new State);
		spAgent agent2 = Agent007::createAgent(factory, "agent2", aState);

		site->setAgent(agent2);

		test_(site->hasAgent());
		test_(site->getAgent() == agent2);
		test_(agent2->getSite() == site);
		test_(site->numOfAdjacentSites() == nSites);
	} catch (Site::invalid_argument_site& e) {
		fail_(string("Shouldn't have thrown an exception. ") + e.what());
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}

	test_(!site->hasAgent()); // Agent is out of scope!

	try {
		spState aState3(new State);
		spAgent agent3 = Agent007::createAgent(factory, "agent3", aState3);

		test_(!site->hasAgent());
		site->setAgent(agent3);
		test_(site->hasAgent());

		spState aState4(new State);
		spAgent agent4 = Agent007::createAgent(factory, "agent4", aState4);

		test_(site->hasAgent());
		site->setAgent(agent4);

		fail_("Should have thrown an exception.");
	} catch (Site::invalid_argument_site&) {
		succeed_();
	} catch (std::exception& e) {
		fail_(string("Unexpected exception. ") + e.what());
	}
}
*/